package com.geronimolive.gcal.client;

import com.geronimolive.gcal.model.Event; 
import com.geronimolive.gcal.ws.client.EventService;
import com.geronimolive.gcal.ws.client.EventServiceServiceLocator;


import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.Calendar;

import javax.xml.rpc.ServiceException;


/**
 * A class to test the event service using 
 * generated JAX-RPC client stubs (i.e. "the easy way").
 * 
 * @author Tom McQueeney
 */
public class EventServiceGenStubClient {

    private final URL serviceURL;
    
    /**
     * This default constructor creates a client that calls the 
     * web service at its default URL (from the WSDL). 
     */
    public EventServiceGenStubClient() {
        serviceURL = null;
    }
    
    /**
     * Creates a client that calls the event web service at the given URL.
     * 
     * @param url
     *            the url for the web service we will invoke.
     * @throws MalformedURLException
     *             if the string url parameter is invalid.
     */
    public EventServiceGenStubClient(String url) 
      throws MalformedURLException 
    {
        serviceURL = new URL(url);
    }

    /**
     * Main method to exercise the web service.
     * If called with a string parameter, it will use that string
     * as the web service's URL.
     * 
     * @param args pass an optional URL as the first element to override
     *        the web service's URL.
     * @throws MalformedURLException if the URL given in <code>args</code> 
     *         is malformed.
     * @throws ServiceException if the web service could not be created.
     * @throws RemoteException if there was a problem invoking the 
     *         remote web service.
     */
    public static void main(String[] args) 
      throws MalformedURLException, ServiceException, RemoteException 
    {
        // Create the time service.
        EventServiceGenStubClient eventServiceClient = newInstance(
            args.length == 0 ? null : args[0]
        );
        EventService ts = eventServiceClient.getEventService();

        // Invoke the service to save a new Event
        Event event = new Event();
        event.setDescription("A test event created by web service");
        event.setStartDateTime(Calendar.getInstance());
        event.setEndDateTime(event.getStartDateTime());
        event.setLink("http://geronimolive.com/");

        System.out.println("Calling web service to save event");
        ts.saveEvent(event);
        System.out.println("Returned from saving event via web service");
        
        // Retrieve and display all existing events from the web service.
        final Event[] events = ts.getEvents();
        System.out.println("Events from the web service:");
        for (int i=0, j=events.length; i < j; i++) {
            System.out.println("Event " + (i+1) + ": with ID " + events[i].getId());
            System.out.println("\tDesc: " + events[i].getDescription());
            Calendar c = events[i].getStartDateTime();
            System.out.println("\tStart date: " + (c == null ? "null" : c.getTime().toString()));
            c = events[i].getEndDateTime();
            System.out.println("\tEnd date: " + (c == null ? "null" : c.getTime().toString()));
            System.out.println("\tURL: " + events[i].getLink());
        }
        
        System.out.println("\nDone.");
    }
    
    /**
     * Creates an instance of this test class using the provided URL
     * to determine whether an instance is created using the default URL
     * (if <code>url</code> is null) or at the given URL.
     * 
     * @param url an optional URL to override the web services's default URL.
     *        The url may be <code>null</code>, in which case the 
     *        web service's default URL will be used.
     * @return an instance of this TimeServiceGenStubClient class.
     * @throws MalformedURLException if <code>url</code> is malformed.
     */
    private static EventServiceGenStubClient newInstance(String url) 
      throws MalformedURLException 
    {
        EventServiceGenStubClient eventServiceTest;
        if (url == null) {
            System.out.println("Getting web service at default URL");
            eventServiceTest = new EventServiceGenStubClient();
        } else {
            System.out.println("Getting web service at URL: " + url);
            eventServiceTest = new EventServiceGenStubClient(url);
        }
        return eventServiceTest;
    }

    /**
     * Return an instance of the TimeService by looking
     * up the service using the service locator.
     * 
     * @return an instance of the TimeService class.
     * @throws ServiceException if the service can't be found or created
     */
    public EventService getEventService() throws ServiceException {
        EventServiceServiceLocator serviceLocator = 
            new EventServiceServiceLocator();
        
        if (serviceURL != null) {
            return serviceLocator.getEventService(serviceURL);
        } else {
            return serviceLocator.getEventService();
        }
    }

}
